from saysynth import Segment, Note, MidiTrack


def test_segment():
    segment = Segment(note="A3", velocity=127, duration=100)
    assert str(segment) == "[[ volm 1.0 ]] 2m {D 100; P 220:0 220:100}"


def test_note():
    note = Note(
        note="C4",
        phoneme="m",
        velocity=127,
        attack=0.1,
        decay=0.2,
        sustain=0.3,
        release=0.5,
        bpm=126,
        count=1 / 2,
        segment_bpm=126,
        segment_count=1 / 32,
    )
    assert note.segments[0] == "[[ volm 0.59 ]] m {D 59.5238; P 261.63:0 261.63:100}"
    assert note.segments[-1] == "[[ volm 0.0 ]] m {D 59.5238; P 261.63:0 261.63:100}"



"""
Play a sequence of commands concurrently via yaml specification
"""
import sys
from pathos.multiprocessing import ProcessingPool as Pool

import yaml
import click

from saysynth.cli.commands import chord, midi, note


TRACK_FUNCS = {
    "chord": chord.run,
    "midi": midi.run,
    "note": note.run,
}

def _run_track_func(kwargs):
    kwargs['exec'] = True
    type = kwargs.get('type', None)
    options = kwargs.get('options', {})
    if type not in TRACK_FUNCS:
        raise ValueError(f'Invalid track type: {type}. Choose from: {",".join(TRACK_FUNCS.keys())}')
    return TRACK_FUNCS.get(type)(**options)

def run(**kwargs):
    seq_config = yaml.safe_load(kwargs['seq_config'])
    tracks = seq_config.pop('tracks', [])
    return Pool(len(tracks)).map(_run_track_func, tracks)


@click.command()
@click.argument("seq_config", type=click.File(), required=True)
def cli(**kwargs):
    return run(**kwargs)